SharedVecPart

Struct SharedVecPart 

pub struct SharedVecPart<T> { /* private fields */ }
A slice-like view into a segment of a SharedVecMut allocation.

It can be read as a slice or mutated in place while the underlying allocation is still owned by the original SharedVecMut.

Dropping a SharedVecPart leaks the underlying allocation. When the panic-on-drop feature is enabled, dropping it will panic:

let mut values = manual_share::SharedVecMut::from_vec(vec![1, 2, 3, 4]);
let mut part = values.split_off(2).unwrap();

// forget SharedVecMut to make sure the panic is not caused by dropping it first.
std::mem::forget(values);

// panic here due to dropping ShareVecPart

Implementations

Source

impl<T> SharedVecPart<T>

pub fn as_slice(&self) -> &[T]

View the part as an immutable slice.

pub fn as_slice_mut(&mut self) -> &mut [T]

View the part as a mutable slice.